import imp
import inspect
import os
import six

__version__ = '0.1.0'

__all__ = [
    'Required',
    'inject',
]


class _Required(object):
    """Object that indicates that a variable must be defined externally."""
    pass
Required = _Required()


class VarNotDefined(Exception):
    """
    Raised if the external config fails to define a variable that the main
    config requires.
    """
    pass


class UnknownVarDefined(Exception):
    """
    Raised if the external config defines a variable that the main config
    hasn't defined.
    """
    pass


def _get(key, default=None):
    """Use it if a setting may not exist, and a default is necessary"""
    return globals().get(key, default)


def inject(path):
    """
    Injects the uppercase global variables in the Python module at the given
    filesystem path into the invoker's global variables.

    Args:
        path (str): The filesystem path to the Python module to import and
            inject into the invoker's global variables.
    """

    if os.path.exists(path):
        if six.PY2:
            loaded_conf = imp.load_source(
                'pyconfimporter.loaded', path)
        else:
            from importlib.machinery import SourceFileLoader
            loaded_conf = SourceFileLoader(
                'pyconfimporter.loaded', path).load_module()

        if six.PY2:
            env = inspect.stack()[1][0].f_globals
        else:
            env = inspect.stack()[1].frame.f_globals

        # copy all uppercase keys from service settings file to this file
        for key in dir(loaded_conf):
            if key.isupper():
                if key not in env:
                    raise UnknownVarDefined(
                        'Unknown variable defined: %s' % key)
                env[key] = getattr(loaded_conf, key)
    else:
        raise IOError('Could not find {}'.format(path))

    for key, val in env.items():
        if key.isupper() and env[key] == Required:
            raise VarNotDefined('Required variable not defined: %s' % key)

    env['get'] = _get



from __future__ import absolute_import, division, print_function, unicode_literals

import os
import unittest

import pyconfimporter


A = 1
B = 2
C = 3
D = None


class TestPyConfImporter(unittest.TestCase):

    def test_pyconfimporter(self):

        path_to_ext_conf = os.path.join(
            os.path.dirname(__file__), 'ext_conf.py')

        # Test basic injection
        pyconfimporter.inject(path_to_ext_conf)

        assert A == -1
        assert D == 4

        # Try injection that doesn't specify required value
        globals()['E'] = pyconfimporter.Required

        with self.assertRaises(pyconfimporter.VarNotDefined) as cm:
            pyconfimporter.inject(path_to_ext_conf)
        assert cm.exception.args[0] == 'Required variable not defined: E'

        # Try a conf that does not exist.
        with self.assertRaises((IOError, OSError)) as cm:
            pyconfimporter.inject('does_not_exist.py')
        assert cm.exception.args[0] == 'Could not find does_not_exist.py'

        # Try a conf that specifies an unknown variable
        path_to_ext_conf2 = os.path.join(
            os.path.dirname(__file__), 'ext_conf2.py')
        with self.assertRaises(pyconfimporter.UnknownVarDefined) as cm:
            pyconfimporter.inject(path_to_ext_conf2)
        assert cm.exception.args[0] == 'Unknown variable defined: F'



from setuptools import setup

import pyconfimporter

install_reqs = [
    'six>=1.3.0',
]

description = """\
Easy way to import an external Python module into an environment.
"""

setup(
    name='pyconfimporter',
    version=pyconfimporter.__version__,
    description=description,
    author='Ken Elkabany',
    author_email='ken@elkabany.com',
    url='https://github.com/braincore/pyconfimporter',
    install_requires=install_reqs,
    py_modules=['pyconfimporter'],
    license='MIT',
    platforms=['CPython 2.7', 'CPython 3.4', 'CPython 3.5'],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        ],
    )
